'use strict';

Object.defineProperty(exports, "__esModule", {
  value: true
});
exports.configureServer = undefined;

var _path = require('path');

var _path2 = _interopRequireDefault(_path);

var _fs = require('fs');

var _fs2 = _interopRequireDefault(_fs);

var _util = require('util');

var _url = require('url');

var _Dr = require('dr-js/library/Dr.node');

var _dev__ = require('./__dev__');

function _interopRequireDefault(obj) { return obj && obj.__esModule ? obj : { default: obj }; }

const utimesAsync = (0, _util.promisify)(_fs2.default.utimes);
const unlinkAsync = (0, _util.promisify)(_fs2.default.unlink);
const readFileAsync = (0, _util.promisify)(_fs2.default.readFile);

const { Format, Module: { BASIC_EXTENSION_MAP }, Time: { getTimestamp } } = _Dr.Common;
const {
  File: { FILE_TYPE, getPathType, createDirectory, getDirectoryContentNameList },
  Resource: { fetch, receiveBufferAsync },
  Server: {
    createServer, createRequestListener,
    Responder: {
      responderEnd, responderEndWithStatusCode,
      responderSendBuffer, responderSendJSON,
      createResponderRouter, createRouteMap, getRouteParam,
      createResponderParseURL,
      createResponderLogRequestHeader, createResponderLogEnd
    }
  }
} = _Dr.Node;

const configureServer = ({ hostname, port, serverLogger, urlSnapshotTask, taskRunner }) => {
  const responderLogEnd = createResponderLogEnd((data, state) => {
    const errorLog = state.error ? ` [ERROR] ${data.finished ? 'finished' : 'not-finished'} ${state.error.stack || state.error.toString()}` : '';
    serverLogger.add(`[END] ${Format.time(data.duration)} ${data.statusCode}${errorLog}`);
  });

  const responderStartTask = (store, inputData) => {
    const task = taskRunner.startTask(inputData);
    serverLogger.add(`[Task|start] end: ${task.id}`);
    return responderSendJSON(store, { object: { taskId: task.id } });
  };
  const responderGetTaskStatus = (store, taskId) => {
    const task = taskRunner.getTaskState(taskId);
    if (!task) return responderSendJSON(store, { object: {} });
    const { error, progress = 1 } = task;
    const taskStatus = { taskId, error: error && error.toString(), progress };
    serverLogger.add(`[Task|status] end: ${JSON.stringify(taskStatus)}`);
    return responderSendJSON(store, { object: taskStatus });
  };
  const responderDeleteTask = (store, taskId) => {
    taskRunner.endTask(taskId);
    serverLogger.add(`[Task|delete] end`);
    return responderEnd(store);
  };


  const { server, start, stop, option } = createServer({ protocol: 'http:', hostname, port });
  server.on('request', createRequestListener({
    responderList: [createResponderLogRequestHeader(data => serverLogger.add(`[REQUEST] ${data.method} ${data.host} ${data.url} ${data.remoteAddress} ${data.remotePort}`)), createResponderParseURL(option), createResponderRouter(createRouteMap([[['/nginx-sync-snapshot', '/nginx-sync-snapshot.png'], 'GET', async store => {
      const { routeFile } = await doSyncTask(store, { serverLogger, urlSnapshotTask, taskRunner });
      return responderEndWithStatusCode(store, { statusCode: 200, headerMap: { 'x-accel-redirect': encodeURI(routeFile) } });
    }], [['/debug-sync-snapshot', '/debug-sync-snapshot.png'], 'GET', async store => {
      const { pathFile } = await doSyncTask(store, { serverLogger, urlSnapshotTask, taskRunner });
      return responderSendBuffer(store, { buffer: await readFileAsync(pathFile), type: BASIC_EXTENSION_MAP.png });
    }], ['/task', 'POST', async store => responderStartTask(store, JSON.parse((await receiveBufferAsync(store.request))))], ['/task/:task-id', 'GET', store => responderGetTaskStatus(store, getRouteParam(store, 'task-id'))], ['/task/:task-id', 'DELETE', store => responderDeleteTask(store, getRouteParam(store, 'task-id'))], ['/debug', 'GET', _dev__.responderSendDebugPage], [['/', '/status'], 'GET', store => {
      const { url: { searchParams } } = store.getState();
      const isVerbose = searchParams.has('verbose');
      const isPretty = isVerbose && searchParams.has('pretty');
      let taskStatusList = taskRunner.getStatus({ isVerbose });
      if (isPretty) taskStatusList = taskStatusList.map(taskStatus => taskStatus.taskStateList.map(([id, state]) => `[${state.error || `${state.progress || 0}%`}] ${id}`));
      responderSendJSON(store, { object: taskStatusList });
    }], ['/favicon.ico', 'GET', _dev__.responderSendFavicon]]))],
    responderEnd: async store => {
      await responderEnd(store);
      await responderLogEnd(store);
    }
  }));
  return { server, start, stop, option };
};

const doSyncTask = async (store, { serverLogger, urlSnapshotTask, taskRunner }) => {
  const { url: { searchParams } } = store.getState();
  const response = await fetch(`${urlSnapshotTask}?${new _url.URLSearchParams({ access_token: searchParams.get('access-token'), screen_cid: searchParams.get('screen-cid') })}`);
  if (!response.ok) {
    response.end();
    throw new Error('failed to get snapshot task data');
  }

  const inputData = await response.json();
  const { taskList, pathOutput, routeOutput } = inputData;
  if (taskList && taskList.length !== 1) throw new Error('expect to get single task taskList');

  const { fileName, fileNamePrefix } = taskList[0];
  if (await getPathType(_path2.default.join(pathOutput, fileName) === FILE_TYPE.File)) {
    const timestamp = getTimestamp();
    await utimesAsync(_path2.default.join(pathOutput, fileName), timestamp, timestamp);
  } else {
    await createDirectory(pathOutput);

    const { id: taskId, promise: taskPromise } = taskRunner.startTask(inputData);
    taskPromise && (await taskPromise);
    const { error } = taskRunner.getTaskState(taskId);
    if (error) throw error;
  }

  await deleteFileWithPrefix(pathOutput, fileNamePrefix, fileName);

  serverLogger.add(`[Task|sync] end: ${fileName}`);

  return {
    routeFile: _path2.default.join('/', routeOutput, fileName),
    pathFile: _path2.default.join(pathOutput, fileName)
  };
};

const deleteFileWithPrefix = async (directoryPath, prefix, except) => {
  const nameList = await getDirectoryContentNameList(directoryPath);
  for (const name of nameList) {
    name !== except && name.startsWith(prefix) && (await unlinkAsync(_path2.default.join(directoryPath.name)));
  }
};

exports.configureServer = configureServer;